"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function throttledQueue(maxRequestsPerInterval, interval, evenlySpaced) {
    if (evenlySpaced === void 0) { evenlySpaced = false; }
    /**
     * If all requests should be evenly spaced, adjust to suit.
     */
    if (evenlySpaced) {
        interval = interval / maxRequestsPerInterval;
        maxRequestsPerInterval = 1;
    }
    if (interval < 200) {
        console.warn('An interval of less than 200ms can create performance issues.');
    }
    var queue = [];
    var lastCalled = Date.now();
    var timeout;
    /**
     * Gets called at a set interval to remove items from the queue.
     * This is a self-adjusting timer, since the browser's setTimeout is highly inaccurate.
     */
    var dequeue = function () {
        var threshold = lastCalled + interval;
        var now = Date.now();
        /**
         * Adjust the timer if it was called too early.
         */
        if (now < threshold) {
            clearTimeout(timeout);
            timeout = setTimeout(dequeue, threshold - now);
            return;
        }
        for (var _i = 0, _a = queue.splice(0, maxRequestsPerInterval); _i < _a.length; _i++) {
            var callback = _a[_i];
            callback.call({});
        }
        lastCalled = Date.now();
        if (queue.length) {
            timeout = setTimeout(dequeue, interval);
        }
        else {
            timeout = undefined;
        }
    };
    return function (fn) { return new Promise(function (resolve, reject) {
        queue.push(function () { return Promise.resolve().then(fn).then(resolve).catch(reject); });
        if (!timeout) {
            timeout = setTimeout(dequeue, interval);
        }
    }); };
}
exports.default = throttledQueue;
//# sourceMappingURL=throttledQueue.js.map